bool AUTORELOAD;

float4 vecSkill41;//LightColor.rgb LightRange
float4 vecSkill45;
float4 vecSkill17;//ClipFar.x Arc.y

float _vLF_LightingMode_var;
float _vLF_SaturationMode_var;

const float small=1.f/255.f;
const float small2=1.f/(1.f-(1.f/255.f));

float4x4 matWorldViewProj;
float4x4 matWorld;
float4x4 matViewInv;
//float4x4 matView;
//float4x4 matViewProj;
float4x4 matProj;

//float4 vecSunDir;		
float4 vecViewPos;
float4 vecViewPort;
//float4 vecViewDir;
//float4 vecTime;

texture mtlSkin1;
texture mtlSkin2;
texture mtlSkin3;

texture entSkin4;
texture entSkin3;


sampler NORMAL_SAMPLER = sampler_state { Texture   = <mtlSkin1>; MinFilter = Point; MagFilter = Point; MipFilter = Point;    AddressU  = clamp; AddressV  = clamp; };
sampler DEPTH_SAMPLER  = sampler_state { Texture   = <mtlSkin2>; MinFilter = Point; MagFilter = Point; MipFilter = Point;    AddressU  = clamp; AddressV  = clamp; };

sampler SHADOW_SAMPLER = sampler_state { Texture   = <entSkin3>; MinFilter = Linear;MagFilter = Linear;MipFilter = Linear;   AddressU  = clamp; AddressV  = clamp; };
sampler COOKIE_SAMPLER = sampler_state { Texture   = <entSkin4>; MinFilter = Point; MagFilter = Linear; MipFilter = Linear;    AddressU  = clamp; AddressV  = clamp; };

float add_zero(float F) { return (F-small)*small2; }
float add_zero2(float F) { return lerp(F,(F-small)*small2,_vLF_LightingMode_var); }
//float add_zero2(float F) { return (F-small*_vLF_LightingMode_var)*(1+small*_vLF_LightingMode_var); }

float3 create_unlike_vector(float3 V)
{
	return cross(float3(V.xy,V.z+1),float3(V.x+1,V.yz));
}

float g_MinVariance=10.f;

float ReduceLightBleeding(float p_max, float Amount)
{
	return saturate((p_max-Amount)/(1-Amount));	
}

float ChebyshevUpperBound(float2 Moments, float t)
{
	float p=(t<=Moments.x);
	
	float Variance=Moments.y-(Moments.x*Moments.x);
	Variance = max(Variance, g_MinVariance);
	
	float d=t-Moments.x;
	float p_max=Variance/(Variance+d*d);
	
	return max(p,p_max);
}

float4 blur_cubemap(sampler CUBE_SAMPLER,float3 TexCoords,float offset, float DistanceToLight)
{
	float count=2.f;
	offset/=count;
	
	float3 off_dir1;
	float3 off_dir2;
	
	off_dir1=normalize(cross(create_unlike_vector(TexCoords),TexCoords));
	off_dir2=normalize(cross(TexCoords,off_dir1));
	
	offset/=max(1,sqrt(DistanceToLight)*0.18f);//reduce light bleeding
	
	off_dir1*=offset;
	off_dir2*=offset;
	
	float4 Value=0;
	
	for(float i=-count;i<=count;i++)
	for(float j=-count;j<=count;j++)
	{		
		float3 newTexCoords=TexCoords + off_dir1*i + off_dir2*j;
		float4 d=texCUBE(CUBE_SAMPLER,newTexCoords);

		Value+=d;
	}
	
	return Value/pow(count*2+1,2);	
}

float genVSM(float3 TexCoords,float3 FaceDir, float DistanceToLight,float BlurOffset,float lbReductionStrength)
{
	float2 Moments=blur_cubemap(SHADOW_SAMPLER,TexCoords,BlurOffset,DistanceToLight).xy;
	//	Moments=texCUBE(SHADOW_SAMPLER,TexCoords);

	float vsm=ChebyshevUpperBound(Moments,DistanceToLight);
	
	return ReduceLightBleeding(vsm,lbReductionStrength);
}

void VS(		
in float4 vPos		   : POSITION,

out float4  pPos        : TEXCOORD1,
out float3  LightColor  : TEXCOORD2,
out float4  WorldPos    : TEXCOORD3,

out float4 Pos	         : POSITION
)
{
	Pos = mul(vPos,matWorldViewProj);
	
	pPos=Pos;
	
	//	vecSkill41.yzw=float3(0,128,192);
	LightColor=vecSkill41.yzw/255.f;
	LightColor*=dot(LightColor,float3(0.3505f,0.2065f,0.443f));
	LightColor*=vecSkill45.z;
	
	float3 LightColor_norm=LightColor.rgb/max(LightColor.r,max(LightColor.g,LightColor.b));
	LightColor_norm=min(LightColor_norm,LightColor);
	
	LightColor=lerp(LightColor_norm,LightColor,_vLF_SaturationMode_var);
	//	LightColor*=dot(LightColor,float3(0.299f,0.587f,0.114f));
	
	
	float3 wPos = mul(vPos,matWorld);

	//	Depth=dot(vecViewDir,wPos - vecViewPos)/_vCameraClipFar_var;
	
	WorldPos.xyz=wPos;
	WorldPos.w=matWorldViewProj[3].w;
	
}

float3 get_WorldPos_from_View(float Depth, float2 Tex)
{
	float2 sTex=float2(Tex.x,1-Tex.y)*2-1;
	
	float2 XY=tan(radians(vecSkill17.y))*Depth*float2(1,vecViewPort.y/vecViewPort.x);

	return mul(float4(XY*sTex,Depth,1),matViewInv).xyz;
}

float2 calc_ScreenPos(float4 pPos)
{
	return (float2(pPos.x,-pPos.y)/pPos.w+vecViewPort.zw)*0.5+0.5;
}

float extBlinn(float NH,float roughness,float fresnel)
{
	float e=pow(2,9*roughness);
	
	float n=1-(1-roughness)*(1-roughness);//approx. normalization 
	
	return pow(NH,e)*n*fresnel;
}

void PS(	
in float4  pPos        : TEXCOORD1,
in float3  LightColor  : TEXCOORD2,
in float4  WorldPos    : TEXCOORD3,

out float4 DIFF :COLOR0
) 
{
	float2 pTex=calc_ScreenPos(pPos);//+0.5*vecViewPort.zw;
	
	float4 D=tex2D(DEPTH_SAMPLER,pTex);
	
	float oDepth=D.x+(D.y/255.f)+(D.z/255.f/255.f);
	
	if(oDepth==1)
	clip(-1);
	
	oDepth*=vecSkill17.x;
	
	//	if(pPos.w-oDepth<0)clip(-1);
	
	float3 wPos=get_WorldPos_from_View(oDepth,pTex);
	
	float3 oPos=wPos-matWorld[3].xyz;

	///////////////////Attenuation
	float att_s=7;
	
	float att_x=dot(oPos,oPos)/(vecSkill41.x*vecSkill41.x);
	
	if(att_x>=1)clip(-1);
	
	float Att=1.f/(att_x*att_s+1.f); 
	
	att_s=1.f/(att_s+1);
	
	Att=(Att-att_s)/(1.f-att_s);
	///////////////////
	
	float4 NM=tex2D(NORMAL_SAMPLER,pTex);
	
	float3 normal=normalize(NM.xyz*2-1);
	
	float3 LightDir=-normalize(oPos);
	
	float3 ViewDir=normalize(vecViewPos.xyz-wPos);
	
	float3 H=normalize(LightDir+ViewDir);
	
	////	float LH=saturate(dot(H,LightDir));
	float NH=saturate(dot(H,normal));
	float LN=saturate(dot(LightDir,normal));
	
	//	Att=saturate(Att*100000);//Debugging
	///////////////////////////////////////
	
	float3 d3=abs(oPos);
	float d=max(d3.x,max(d3.y,d3.z));

	float3 CubeFaceDir=sign(max(0,sign(d3-d)+0.5));
	float UnitCubeLength=abs(dot(CubeFaceDir,LightDir));
	float3 LVc=-LightDir/UnitCubeLength;
	
	float VSM=genVSM(LVc,CubeFaceDir,d-1,0.02f,0.3f);
	//	VSM=1;
	matWorld[3]=0;
	float3 cookie=texCUBE(COOKIE_SAMPLER,mul(LVc,matWorld)).rgb;
	LightColor*=cookie;
	///////////////////////////////////////
	DIFF.rgb=Att*LN*LightColor*VSM;
	DIFF.a=extBlinn(NH,add_zero(D.w),add_zero(NM.w)*1)*saturate(LN*Att*3)*VSM*saturate(vecSkill45.z);
	
	//	DIFF.rgb=texCUBE(SHADOW_SAMPLER,LVc).r*0.001;
	////	DIFF.rgb=tex2D(SHADOW_SAMPLER,pTex).w*0.1;
	////DIFF.rgb=LVc;
	//	DIFF.rgb=VSM;
}

technique Shading
{
	pass P0
	{		
		cullmode=cw;
		alphablendenable=true;
		zfunc=always;
		srcblend=one;
		destblend=one;
		
		VertexShader = compile vs_3_0 VS();
		PixelShader  = compile ps_3_0 PS();
	}
}